from starlette.middleware.base import (
    RequestResponseEndpoint,
)
from starlette.responses import Response
from starlette_context.middleware import ContextMiddleware
from starlette_context import _request_scope_context_storage

from fastapi import Request

from examples.config import (
    TEST_DEFAULT_USER_ID,
    TEST_DEFAULT_PLATFORM_ID,
)


class AuthedMiddleware(ContextMiddleware):
    def required_user_id(self) -> bool:
        """
        Example method so we can easily mock it in tests environment
        :return:
        """
        return False

    def required_platform_id(self) -> bool:
        """
        Example method so we can easily mock it in tests environment
        :return:
        """
        return False

    async def authenticate(self):
        self.token = _request_scope_context_storage.set(
            dict(
                user_id=TEST_DEFAULT_USER_ID,
                platform_id=TEST_DEFAULT_PLATFORM_ID,
            )
        )

    async def authorize(self):
        context = _request_scope_context_storage.get()
        context['user_id_filter_required'] = self.required_user_id()
        context['platform_id_filter_required'] = self.required_platform_id()
        self.token = _request_scope_context_storage.set(context)

    async def dispatch(
        self, request: Request, call_next: RequestResponseEndpoint
    ) -> Response:
        try:
            # Authentication and authorization goes here!
            await self.authenticate()
            await self.authorize()
            response = await call_next(request)
            for plugin in self.plugins:
                await plugin.enrich_response(response)

        finally:
            _request_scope_context_storage.reset(self.token)

        return response



import datetime as dt

from fastapi.responses import JSONResponse as Response
from fastapi import Request
from fast_agave.filters import generic_query
from ..models import Account as AccountModel
from ..validators import (
    AccountQuery,
    AccountRequest,
    AccountUpdateRequest,
    AccountResponse,
)
from .base import app


@app.resource('/accounts')
class Account:
    model = AccountModel
    query_validator = AccountQuery
    update_validator = AccountUpdateRequest
    get_query_filter = generic_query

    _response_model = AccountResponse

    @staticmethod
    async def create(request: AccountRequest) -> Response:
        """This is the description for openapi"""
        account = AccountModel(
            name=request.name,
            user_id=app.current_user_id,
            platform_id=app.current_platform_id,
        )
        await account.async_save()
        return Response(content=account.to_dict(), status_code=201)

    @staticmethod
    async def update(
        account: AccountModel,
        request: AccountUpdateRequest,
    ) -> Response:
        account.name = request.name
        await account.async_save()
        return Response(content=account.to_dict(), status_code=200)

    @staticmethod
    async def delete(account: AccountModel, _: Request) -> Response:
        account.deactivated_at = dt.datetime.utcnow().replace(microsecond=0)
        await account.async_save()
        return Response(content=account.to_dict(), status_code=200)



import random
from fast_agave.tasks.sqs_tasks import task
from fast_agave.exc import RetryTask


# Esta URL es solo un mock de la queue.
# Debes reemplazarla con la URL de tu queue
QUEUE_URL = 'http://127.0.0.1:4000/123456789012/core.fifo'


class YouCanTryAgain(Exception):
    ...


def test_your_luck(message):
    value = random.uniform(100)
    if 0 < value <= 33:
        print('you are lucky!', message)
    elif 33 < value <= 66:
        raise YouCanTryAgain
    else:
        raise Exception('game over! :(')


@task(queue_url=QUEUE_URL, region_name='us-east-1', max_retries=2)
async def dummy_retry_task(message) -> None:
    try:
        test_your_luck(message)
    except YouCanTryAgain:
        raise RetryTask



from mongoengine import StringField
from mongoengine_plus.aio import AsyncDocument
from mongoengine_plus.models import BaseModel

from mongoengine_plus.models.helpers import uuid_field


class File(BaseModel, AsyncDocument):
    id = StringField(primary_key=True, default=uuid_field('TR'))
    user_id = StringField(required=True)
    name = StringField(required=True)
